package com.sundata.internalevaluation.calc.calcUnit;

import com.sundata.internalevaluation.calc.model.CalcResult;
import com.sundata.internalevaluation.calc.model.CalcUnit;
import com.sundata.internalevaluation.calc.model.finals.CalcType;
import com.sundata.internalevaluation.configuration.model.SysReqRule;
import com.sundata.internalevaluation.configuration.model.SysReqRules;
import com.sundata.internalevaluation.configuration.service.SysReqRuleService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * Created by IntelliJ IDEA.
 *
 * @author JoeLazy
 * @date 2025-02-12 09:32:30
 * @description: 规则集计算单元
 */

public class RulesCalcUnit extends CalcUnit {

    private static final Logger log = LoggerFactory.getLogger(RulesCalcUnit.class);

    /**
     * 规则集实体
     */
    private final SysReqRules reqRules;


    private final SysReqRuleService sysReqRuleService;


    /**
     * 创建数据单元的绝对对象，对象必须包含如下参数
     *
     * @param calcCode    计算对象编号
     * @param calcName    计算对象名称
     * @param initContext 计算单元初始化参数
     */
    public RulesCalcUnit(String calcCode, String calcName, Map<String, Object> initContext, SysReqRules reqRules, SysReqRuleService sysReqRuleService) {
        super(calcCode, calcName, CalcType.RULES, initContext);

        this.reqRules = reqRules;

        this.sysReqRuleService = sysReqRuleService;
    }

    /**
     * 判断是否已经计算过数据了
     *
     * @param calculateInstanceNumber 计算流水号
     * @return 是否计算过 true 计算过 false 没有计算过
     */
    @Override
    public boolean isCalcFinished(String calculateInstanceNumber) {
        // 计算是否计算过

        return false;
//        return DataImages.rulesCalcUnitHashMap.containsKey(calculateInstanceNumber)
//                && DataImages.rulesCalcUnitHashMap.get(calculateInstanceNumber)
//                .stream().anyMatch(a -> a.getCalcCode().equals(this.getCalcCode()));
    }

    /**
     * 初始化计算结果的方法，
     * 如果已经计算过，在实现过程中，应当在此方法中根据计算流水号重新初始化 resultContext 结果对象，为其他依赖对象做准备
     * 若明明计算过本单元但再次计算时没有初始化该对象，则计算依赖出现问题无法定位与处理
     *
     * @param calculateInstanceNumber 计算流水号
     */
    @Override
    public void initResultContext(String calculateInstanceNumber) {
        // TODO 初始化
//        List<RulesCalcUnit> rulesCalcUnits = new ArrayList<>();
////        List<RulesCalcUnit> rulesCalcUnits = DataImages.rulesCalcUnitHashMap.get(calculateInstanceNumber);
//        // 筛选并查找对象，如果找不到则报错
//        if (rulesCalcUnits.stream().noneMatch(a -> a.getCalcCode().equals(this.getCalcCode()))) {
//            throw new CalcException(calculateInstanceNumber, StrUtil.format("无法找到已计算完成的结果，计算单元编号：{}，计算流水号为：{}。", this.getCalcCode(), calculateInstanceNumber));
//        }
//        this.setResultContext(
//                rulesCalcUnits.stream()
//                        .filter(a -> a.getCalcCode().equals(this.getCalcCode()))
//                        .findFirst().get().getResultContext()
//        );
    }

    private List<SysReqRule> sysReqRuleList = new ArrayList<>();

    /**
     * 根据节点配置获取源节点；
     *
     * @return 所有源头节点
     */
    @Override
    public List<CalcUnit> getSourceCalcUnits() {
        List<SysReqRule> sysReqRules = sysReqRuleService.selectRuleListByRulesNo(reqRules.getRulesNo());
        return sysReqRules.stream()
                .map(rule -> new RuleCalcUnit(rule.getRuleNo(), rule.getRuleName(),
                        CalcType.RULE, Map.of(), rule)).collect(Collectors.toList());

    }

    /**
     * 计算之后的方法，可实现为空
     *
     * @param context 计算参数过程数据
     */
    @Override
    public void afterCalc(Map<String, Object> context) {
        log.debug("计算之后的参数结构：{}", context);
    }

    /**
     * 计算之前，可实现空
     *
     * @param context 计算参数过程数据
     */
    @Override
    public void beforeCalc(Map<String, Object> context) {
        log.debug("计算之前的参数结构：{}", context);
    }


    /**
     * 必须实现的主体计算内容
     *
     * @param context       节点计算参数清单
     * @param sourceResults 源头计算节点的结果
     */
    @Override
    public void calc(final CalcResult<String, Object> thisResult,
                     String calculateInstanceNumber,
                     Map<String, Object> context,
                     Map<CalcUnit, CalcResult<String, Object>> sourceResults) {

        log.debug("计算{}：编号:{}，名称:{}，流水号:{}", getCalcType().getName(),getCalcCode(),getCalcName(),calculateInstanceNumber);

        // 规则结果为 true 的规则
        List<Map.Entry<CalcUnit, CalcResult<String, Object>>> resIsTrue = sourceResults.entrySet().stream()
                .filter(entry -> entry.getValue().get(entry.getKey().getCalcCode()).equals(true))
                .collect(Collectors.toList());

        //
        Map<String, List<SysReqRule>> ruleListMap = resIsTrue.stream()
                .collect(Collectors.groupingBy(e1 -> {
                    SysReqRule o = (SysReqRule) e1.getValue().get("entity");
                    return o.getRuleResult();
                }, Collectors.mapping(e ->
                        (SysReqRule) e.getValue().get("entity"), Collectors.toList()
                )));


        List<SysReqRule> interceptRuleList = ruleListMap.get("intercept");
        List<SysReqRule> warnRuleList = ruleListMap.get("warn");

        if (interceptRuleList != null && !interceptRuleList.isEmpty()) {
            // 存在拦截的规则
            // TODO  拦截处理
            List<String> resList = interceptRuleList.stream().map(rule -> String.format("【%s】%s 触发 %s", rule.getRuleNo(), rule.getRuleName(), "拦截")).collect(Collectors.toList());
            thisResult.put(getCalcCode(),resList);
            return;
        }



        if (warnRuleList != null && !warnRuleList.isEmpty()) {
            // 存在预警的规则
            // TODO  预警处理
            List<String> resList = warnRuleList.stream().map(rule -> String.format("【%s】%s 触发 %s", rule.getRuleNo(), rule.getRuleName(), "预警")).collect(Collectors.toList());
            thisResult.put(getCalcCode(),resList);
            return;
        }


        // 无拦截和预警
        thisResult.put(this.getCalcCode(), "无拦截");

    }
}
